# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""OAuth 2.0 Provider."""

from functools import wraps

from flask import (
    Blueprint,
    abort,
    current_app,
    jsonify,
    redirect,
    render_template,
    request,
)
from flask_login import login_required
from oauthlib.oauth2.rfc6749.errors import InvalidClientError, OAuth2Error

from ..models import Client
from ..provider import oauth2
from ..proxies import current_oauth2server

blueprint = Blueprint(
    "invenio_oauth2server",
    __name__,
    url_prefix="/oauth",
    static_folder="../static",
    template_folder="../templates",
)


def error_handler(f):
    """Handle uncaught OAuth errors."""

    @wraps(f)
    def decorated(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except OAuth2Error as e:
            # Only FatalClientError are handled by Flask-OAuthlib (as these
            # errors should not be redirect back to the client - see
            # http://tools.ietf.org/html/rfc6749#section-4.2.2.1)
            if hasattr(e, "redirect_uri"):
                return redirect(e.in_uri(e.redirect_uri))
            else:
                return redirect(e.in_uri(oauth2.error_uri))

    return decorated


#
# Views
#
@blueprint.route("/authorize", methods=["GET", "POST"])
@login_required
@error_handler
@oauth2.authorize_handler
def authorize(*args, **kwargs):
    """View for rendering authorization request."""
    if request.method == "GET":
        client = Client.query.filter_by(client_id=kwargs.get("client_id")).first()

        if not client:
            abort(404)

        scopes = current_oauth2server.scopes
        ctx = dict(
            client=client,
            oauth_request=kwargs.get("request"),
            scopes=[scopes[x] for x in kwargs.get("scopes", [])],
        )
        return render_template("invenio_oauth2server/authorize.html", **ctx)

    confirm = request.form.get("confirm", "no")
    return confirm == "yes"


@blueprint.route(
    "/token",
    methods=[
        "POST",
    ],
)
@oauth2.token_handler
def access_token():
    """Token view handles exchange/refresh access tokens."""
    client = Client.query.filter_by(client_id=request.form.get("client_id")).first()

    if not client:
        abort(404)

    if not client.is_confidential and "client_credentials" == request.form.get(
        "grant_type"
    ):
        error = InvalidClientError()
        response = jsonify(dict(error.twotuples))
        response.status_code = error.status_code
        abort(response)

    # Return None or a dictionary. Dictionary will be merged with token
    # returned to the client requesting the access token.
    # Response is in application/json
    return None


@blueprint.route("/errors")
def errors():
    """Error view in case of invalid oauth requests."""
    from oauthlib.oauth2.rfc6749.errors import raise_from_error

    try:
        error = None
        raise_from_error(request.values.get("error"), params=dict())
    except OAuth2Error as raised:
        error = raised
    return render_template("invenio_oauth2server/errors.html", error=error)


@blueprint.route("/ping", methods=["GET", "POST"])
@oauth2.require_oauth()
def ping():
    """Test to verify that you have been authenticated."""
    return jsonify(dict(ping="pong"))


@blueprint.route("/info")
@oauth2.require_oauth("test:scope")
def info():
    """Test to verify that you have been authenticated."""
    if current_app.testing or current_app.debug:
        return jsonify(
            dict(
                user=request.oauth.user.id,
                client=request.oauth.client.client_id,
                scopes=list(request.oauth.scopes),
            )
        )
    else:
        abort(404)


@blueprint.route("/invalid")
@oauth2.require_oauth("invalid_scope")
def invalid():
    """Test to verify that you have been authenticated."""
    if current_app.testing or current_app.debug:
        # Not reachable
        return jsonify(dict(ding="dong"))
    else:
        abort(404)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test helper functions."""

from __future__ import absolute_import, print_function

from urllib.parse import parse_qs, urlparse, urlunparse

from flask import Blueprint, abort, jsonify, request, session, url_for
from flask_oauthlib.client import OAuth, prepare_request


def patch_request(app):
    """Patch the request."""
    test_client = app.test_client()

    def make_request(uri, headers=None, data=None, method=None):
        uri, headers, data, method = prepare_request(uri, headers, data, method)
        if not headers and data is not None:
            headers = {"Content-Type": " application/x-www-form-urlencoded"}

        # test client is a `werkzeug.test.Client`
        parsed = urlparse(uri)
        uri = "%s?%s" % (parsed.path, parsed.query)
        resp = test_client.open(
            uri,
            headers=headers,
            data=data,
            method=method,
        )
        # for compatible
        resp.code = resp.status_code
        return resp, resp.data

    return make_request


def parse_redirect(location, parse_fragment=False):
    """Parse a redirect."""
    parse_result = urlparse(location)
    return (
        urlunparse(
            (parse_result.scheme, parse_result.netloc, parse_result.path, "", "", "")
        ),
        parse_qs(parse_result.fragment if parse_fragment else parse_result.query),
    )


def login(test_client, email="info@inveniosoftware.org", password="tester"):
    """Login the test client."""
    return test_client.post(
        url_for("security.login"),
        data={
            "email": email,
            "password": password,
        },
    )


def create_oauth_client(app, name, **kwargs):
    """Helper function to create a OAuth2 client to test an OAuth2 provider."""
    blueprint = Blueprint("oauth2test", __name__, template_folder="templates")

    default = dict(
        consumer_key="confidential",
        consumer_secret="confidential",
        request_token_params={"scope": "test:scope"},
        request_token_url=None,
        access_token_method="POST",
        access_token_url="/oauth/token",
        authorize_url="/oauth/authorize",
        content_type="application/json",
    )
    default.update(kwargs)

    oauth = OAuth(app)
    remote = oauth.remote_app(name, **default)

    @blueprint.route("/oauth2test/login")
    def login():
        return remote.authorize(
            callback=url_for("oauth2test.authorized", _external=True)
        )

    @blueprint.route("/oauth2test/logout")
    def logout():
        session.pop("confidential_token", None)
        return "logout"

    @blueprint.route("/oauth2test/authorized")
    @remote.authorized_handler
    def authorized(resp):
        if resp is None:
            return "Access denied: error=%s" % (request.args.get("error", "unknown"))
        if isinstance(resp, dict) and "access_token" in resp:
            session["confidential_token"] = (resp["access_token"], "")
            return jsonify(resp)
        return str(resp)

    def get_test(test_url):
        if "confidential_token" not in session:
            abort(403)
        else:
            ret = remote.get(test_url)
            if ret.status != 200:
                return abort(ret.status)
            return ret.raw_data

    @blueprint.route("/oauth2test/test-ping")
    def test_ping():
        return get_test(url_for("invenio_oauth2server.ping"))

    @blueprint.route("/oauth2test/test-info")
    def test_info():
        return get_test(url_for("invenio_oauth2server.info"))

    @blueprint.route("/oauth2test/test-invalid")
    def test_invalid():
        return get_test(url_for("invenio_oauth2server.invalid"))

    @remote.tokengetter
    def get_oauth_token():
        return session.get("confidential_token")

    app.register_blueprint(blueprint)

    return remote
